/* audisp-cef.c --
 * Copyright (c) 2014 Mozilla Corporation.
 * Portions Copyright 2008 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *   Guillaume Destuynder <gdestuynder@mozilla.com>
 *   Steve Grubb <sgrubb@redhat.com>
 *	 Tom Stage <tom.stage@arama-consult.com>
 *
 * Resources:
 *		https://github.com/gdestuynder/audisp-json/blob/master/audisp-json.c - Look at this and see if we can use anything from it
 *		https://access.redhat.com/documentation/en-US/Red_Hat_Enterprise_Linux/6/html/Security_Guide/sec-Audit_Record_Types.html
 *		https://access.redhat.com/documentation/en-US/Red_Hat_Enterprise_Linux/6/html/Security_Guide/app-Audit_Reference.html
 *		https://github.com/linux-audit/audit-documentation/blob/master/specs/fields/field-dictionary.csv
 *		https://github.com/linux-audit/audit-documentation/wiki/SPEC-Audit-Event-Parsing-Library
 *		https://github.com/linux-audit/audit-documentation/wiki/SPEC-Audit-Event-Enrichment
 *		http://stackoverflow.com/questions/35821628/getpwuid-r-and-getgrgid-r-are-slow-how-can-i-cache-them
 *		http://pubs.opengroup.org/onlinepubs/009695399/functions/getpwuid.html
 *		http://man7.org/linux/man-pages/man3/auparse_get_milli.3.html
 *		https://www.tutorialspoint.com/cplusplus/cpp_classes_objects.htm - Consider making the cef_msg into an object and see if we can work better with that
 *		http://www.cplusplus.com/forum/beginner/95292/ - Check the if a var is equal to
 *
 * TODO:
 *		Implement the deviceFacility CEF field, and consider moving account_name from CS2 label and field and use this for Facility as the syslog events are doing
 *
 */

#include <stdio.h>
#include <signal.h>
#include <syslog.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <pwd.h>
#include <netdb.h>
#include <sys/stat.h>
#include <time.h>
#include "libaudit.h"
#include "auparse.h"
#include "cef-config.h"

#define CONFIG_FILE "/etc/audisp/audisp-cef.conf"
//This is the maximum arg len for commands before truncating. Syslog often will otherwise truncate the msg.
#define MAX_ARG_LEN 512
#define MAX_ATTR_SIZE 1023
#define MAX_EXTRA_ATTR_SIZE 128
#define BUF_SIZE 32
//Bump when the message is modified
#define CEF_AUDIT_MESSAGE_VERSION 2

//EXTS
#ifndef PROGRAM_NAME
#define PROGRAM_NAME "audisp-cef"
#endif
//EXTS
extern int h_errno;

static volatile int stop = 0;
static volatile int hup = 0;
static cef_conf_t config;
static char *hostname = NULL;
static auparse_state_t *au = NULL;
static int machine = -1;

typedef struct  ll {
        char val[1024];
        struct ll *next;
} attr_t;

struct cef_msg_type {
char    *hdr;
char    *type;
char    *app;
int     version;
char    *msgname;
char    *msgdesc;
int     severity;
struct  ll *attr;
time_t  au_time;
};

struct auditd_cef_map_type {
char    *pid;
char    *uid;
char    *duser;
char    *auid;
char    *auid_user;
char    *suser;
char    *ses;
char    *msg;
char    *acct;
char    *exe;
char    *hostname;
char    *addr;
char    *terminal;
char    *res;
char    *res_outcome;
char    *op;
char    *id;
char    *cmd;
char    *fp;
char    *direction;
char    *spid;
char    *suid;
char    *kind;
char    *rport;
char    *laddr;
char    *lport;
char    *cipher;
char    *ksize;
char    *old_auid;
char    *new_auid;
char    *old_ses;
char    *new_ses;
char    *subj;
char    *ver;
char    *format;
char    *key;
char    *list;
char    *audit_backlog_limit;
char    *old;
char    *cwd;
char    *name;
char    *inode;
char    *ppid;
char	*pproc;
char    *tty;
char    *gid;
char	*gid_name;
char	*grp;
char	*new_group;
char    *euid;
char    *fsuid;
char    *egid;
char    *sgid;
char    *fsgid;
char    *dev;
char    *mode;
char    *ouid;
char    *ogid;
char    *rdev;
char    *fname;
char    *error;
char    *srcname;
char    *flags;
char    *comm;
char    *profile;
char    *info;
};

static void handle_event(auparse_state_t *au, auparse_cb_event_t cb_event_type, void *user_data);

static void term_handler( int sig )
{
        stop = 1;
}

static void hup_handler( int sig )
{
        hup = 1;
}

static void reload_config(void)
{
        hup = 0;
}

/* find string distance from *in until char c is reached */
unsigned int strstok(char *in, char c)
{
        unsigned int slen, len = 0;

        if (in == NULL)
                return len;

        slen = strlen(in);

        while (in[len] != c && len <= slen)
                len++;
        len++;
        return len;
}

int main(int argc, char *argv[])
{
        char tmp[MAX_AUDIT_MESSAGE_LENGTH];
        struct sigaction sa;
        struct hostent *ht;
        char nodename[64];

        sa.sa_flags = 0;
        sigemptyset(&sa.sa_mask);
        sa.sa_handler = term_handler;
        sigaction(SIGTERM, &sa, NULL);
        sa.sa_handler = hup_handler;

        if (load_config(&config, CONFIG_FILE)) {
                fprintf(stderr, "FATAL: Could not read configuration file: %s", CONFIG_FILE);
                return 1;
        }

        openlog("audisp-cef", LOG_CONS, config.facility);

        if (gethostname(nodename, 63)) {
                snprintf(nodename, 10, "localhost");
        }
        nodename[64] = '\0';
        ht = gethostbyname(nodename);
        if (ht == NULL) {
                hostname = strdup("localhost");
                syslog(LOG_ALERT,
                        "gethostbyname could not find machine hostname, please fix this. Using %s as fallback. Error: %s",
                        hostname, hstrerror(h_errno));
        } else {
                hostname = strdup(ht->h_name);
        }

        au = auparse_init(AUSOURCE_FEED, 0);
        if (au == NULL) {
                syslog(LOG_ERR, "could not initialize auparse");
                free_config(&config);
                return -1;
        }

        machine = audit_detect_machine();
        if (machine < 0)
                return -1;

        auparse_add_callback(au, handle_event, NULL, NULL);

		syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|audisp-cef|Triggered when the Audisp CEF Plugin is loaded|1|msg=audisp-cef plugin loaded",CEF_AUDIT_MESSAGE_VERSION);
		syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|audisp-cef|Triggered when the Audisp CEF Plugin is loaded|1|msg=audisp-cef remote_server is: %s:%d \n",CEF_AUDIT_MESSAGE_VERSION,config.remote_server,config.port);
		syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|audisp-cef|Triggered when the Audisp CEF Plugin is loaded|1|msg=audisp-cef facility is: %d \n",CEF_AUDIT_MESSAGE_VERSION,config.facility);
        do {
                if (hup)
                        reload_config();

                while (fgets_unlocked(tmp, MAX_AUDIT_MESSAGE_LENGTH, stdin) &&
                                                        hup==0 && stop==0)
                        auparse_feed(au, tmp, strnlen(tmp, MAX_AUDIT_MESSAGE_LENGTH));

                if (feof(stdin))
                        break;
        } while (stop == 0);

		syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|audisp-cef|Triggered when the Audisp CEF Plugin is unloaded|1|msg=audisp-cef plugin unloaded",CEF_AUDIT_MESSAGE_VERSION);
        closelog();
        auparse_flush_feed(au);
        auparse_destroy(au);
        free_config(&config);

        return 0;
}

int add_extra_record(auparse_state_t *au, char *extra, char *attr)
{
        size_t len = strlen(extra);
        size_t attr_len = strlen(attr) ? MAX_EXTRA_ATTR_SIZE:MAX_EXTRA_ATTR_SIZE;

        if ((len+attr_len) > MAX_ATTR_SIZE)
                return 1;
        snprintf(extra+len, attr_len, " %s\\=%s", attr, auparse_find_field(au, attr));
        return 0;
}

/*
 * This function seeks to the specified record returning its type on succees
 */
static int goto_record_type(auparse_state_t *au, int type)
{
        int cur_type;

        auparse_first_record(au);
        do {
                cur_type = auparse_get_type(au);
                if (cur_type == type) {
                        auparse_first_field(au);
                        return type;  // Normal exit
                }
        } while (auparse_next_record(au) > 0);

        return -1;
}

/* Removes quotes
 * Remove  CR and LF
 * @const char *in: if NULL, no processing is done.
 */
char *unescape(const char *in)
{
        if (in == NULL)
                return NULL;

        char *dst = (char *)in;
        char *s = dst;
        char *src = (char *)in;
        char c;

        while ((c = *src++) != '\0') {
                if ((c == '"') || (c == '\n') || (c == '\r') || (c == '\t')
                                || (c == '\b') || (c == '\f') || (c == '\\'))
                        continue;
                *dst++ = c;
        }
        *dst++ = '\0';
        return s;
}

attr_t *cef_add_attr(attr_t *list, const char *st, const char *val)
{
        attr_t *new;

        if (val == NULL)
                        return list;
        if (strstr(val, "(null)") != NULL)
                        return list;

        new = malloc(sizeof(attr_t));
        snprintf(new->val, MAX_ATTR_SIZE, "%s%s ", st, unescape(val));
        new->next = list;
        return new;
}

char *get_username(int uid)
{
        size_t bufsize;
        char *buf;
        char *name;
        struct passwd pwd;
        struct passwd *result;

        bufsize = sysconf(_SC_GETPW_R_SIZE_MAX);
        if (bufsize == -1)
                        bufsize = 16384;
        buf = (char *)alloca(bufsize);

        if (uid == -1) {
                return NULL;
        }
        if (getpwuid_r(uid, &pwd, buf, bufsize, &result) != 0) {
                return NULL;
        }
        if (result == NULL) {
                return NULL;
        }
        name = strdupa(pwd.pw_name);
        return name;
}

/*
char *get_effective_username(int euid)
{
        size_t bufsize;
        char *buf;
        char *name;
        struct passwd pwd;
        struct passwd *result;

        bufsize = sysconf(_SC_GETPW_R_SIZE_MAX);
        if (bufsize == -1)
                        bufsize = 16384;
        buf = (char *)alloca(bufsize);

        if (euid == -1) {
                return NULL;
        }
        if (geteuid_r(euid, &pwd, buf, bufsize, &result) != 0) {
                return NULL;
        }
        if (result == NULL) {
                return NULL;
        }
        name = strdupa(pwd.pw_name);
        return name;
}

*/
char *get_groupname(int gid)
{
        size_t bufsize;
        char *buf;
        char *name;
        struct passwd pwd;
        struct passwd *result;

        bufsize = sysconf(_SC_GETPW_R_SIZE_MAX);
        if (bufsize == -1)
                        bufsize = 16384;
        buf = (char *)alloca(bufsize);

        if (gid == -1) {
                return NULL;
        }
        if (getgrgid_r(gid, &pwd, buf, bufsize, &result) != 0) {
                return NULL;
        }
        if (result == NULL) {
                return NULL;
        }
        name = strdupa(pwd.pw_name);
        return name;
}

char *get_proc_name(int pid)
{
        char p[1024];
        static char proc[64];
        FILE *fp;
        snprintf(p, 512, "/proc/%d/status", pid);
        fp = fopen(p, "r");
        if (fp) {
                fscanf(fp, "Name: %63s", proc);
                fclose(fp);
        } else
                return NULL;
        return proc;
}

void cef_del_attrs(attr_t *head)
{
        attr_t *prev;
        while (head) {
                prev = head;
                head = head->next;
                free(prev);
        }
}

void syslog_cef_msg(struct cef_msg_type cef_msg)
{
        attr_t *head = cef_msg.attr;
        attr_t *prev;
        char msg[1500];

        snprintf(msg, 1500, "%s|%s|%s|%u|%s|%s|%u|end=%ld000 ", cef_msg.hdr, cef_msg.type, cef_msg.app,
                cef_msg.version, cef_msg.msgname, cef_msg.msgdesc, cef_msg.severity, cef_msg.au_time);
        while (head) {
                        snprintf(msg+strlen(msg), 1500, "%s", head->val);
                        prev = head;
                        head = head->next;
                        free(prev);
        }
		        syslog(LOG_INFO, "%s", msg);
}

static void handle_event(auparse_state_t *au, auparse_cb_event_t cb_event_type, void *user_data)
{
        int type, rc, num=0;
        time_t au_time;

        struct cef_msg_type cef_msg = {
                .hdr            = "CEF:0",
                .type           = "Unix",
                .app            = "auditd",
                .version        = CEF_AUDIT_MESSAGE_VERSION,
                .severity       = 3,
        };

        const char *cwd = NULL;
		const char *argc = NULL;
		const char *cmd = NULL;
        const char *sys;
        const char *syscall = NULL;
		//EXTS
		const char *path = NULL;
		const char *dev = NULL;
		const char *user_acct = NULL;
		const char *user_auth = NULL;
		const char *user_avc = NULL;
		const char *user_chauthtok = NULL;
		const char *user_cmd = NULL;
		const char *user_end = NULL;
		const char *user_err = NULL;
		const char *user_login = NULL;
		const char *user_logout = NULL;
		const char *user_mgmt = NULL;
		const char *user_start = NULL;
		const char *user_tty = NULL;
		const char *add_group = NULL;
		const char *add_user = NULL;
		const char *del_group = NULL;
		const char *del_user = NULL;
		const char *chgrp_id = NULL;
		const char *chuser_id = NULL;
		const char *config_change = NULL;
		const char *cred_acq = NULL;
		const char *cred_disp = NULL;
		const char *cred_refr = NULL;
		const char *crypto_failure_user = NULL;
		const char *crypto_key_user = NULL;
		const char *crypto_login = NULL;
		const char *crypto_logout = NULL;
		const char *crypto_param_change_user = NULL;
		const char *crypto_replay_user = NULL;
		const char *crypto_session = NULL;
		const char *crypto_test_user = NULL;
		const char *eoe = NULL;
		const char *tty = NULL;
		const char *trusted_app = NULL;
		const char *test = NULL;
		const char *system_shutdown = NULL;
		const char *system_runlevel = NULL;
		const char *system_boot = NULL;
		const char *service_start = NULL;
		const char *service_stop = NULL;
		const char *obj_pid = NULL;
		const char *login = NULL;
		const char *kernel = NULL;
		const char *kernel_other = NULL;
		const char *grp_auth = NULL;
		const char *daemon_abort = NULL;
		const char *daemon_accept = NULL;
		const char *daemon_close = NULL;
		const char *daemon_config = NULL;
		const char *daemon_end = NULL;
		const char *daemon_resume = NULL;
		const char *daemon_rotate = NULL;
		const char *daemon_start = NULL;
        //EXTS
		char fullcmd[MAX_ARG_LEN+1] = "\0";
		char fullcmdt[5] = "No\0";
        char extra[MAX_ARG_LEN] = "\0";

        char f[8];
        int len, tmplen;
        int argcount, i, subsys;
        int havecef = 0;
		int promisc; //EXTS

        if (cb_event_type != AUPARSE_CB_EVENT_READY)
                return;

        while (auparse_goto_record_num(au, num) > 0) {
                extra[0] = '\0';
                type = auparse_get_type(au);
                rc = 0;
                auparse_first_field(au);
                cef_msg.au_time = auparse_get_time(au);
				/*
				 * Look into this!!!!
				*/
				//cef_msg.au_time_milli = auparse_get_milli(au);

                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Auditd Type is: %i",CEF_AUDIT_MESSAGE_VERSION, type); //EXTS
                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Auditd 1st field is: %i",CEF_AUDIT_MESSAGE_VERSION, auparse_first_field(au)); //EXTS
                switch (type) {
                        case AUDIT_AVC:
                                argc = auparse_find_field(au, "apparmor");
                                if (!argc)
                                        return;

                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside AUDIT_AVC",CEF_AUDIT_MESSAGE_VERSION); //EXTS
                                havecef = 1;
                                cef_msg.msgname = "AVC_APPARMOR";

                                /*
								cef_msg.attr = cef_add_attr(cef_msg.attr, "cs1Label=Result cs1=", auparse_get_field_str(au));
                                goto_record_type(au, type);

                                cef_msg.msgdesc = unescape(auparse_find_field(au, "info"));
                                goto_record_type(au, type);

                                cef_msg.attr = cef_add_attr(cef_msg.attr, "cs2Label=Operation cs2=", auparse_find_field(au, "operation"));
                                goto_record_type(au, type);

                                cef_msg.attr = cef_add_attr(cef_msg.attr, "cs3Label=Profile cs3=", auparse_find_field(au, "profile"));
                                goto_record_type(au, type);

                                cef_msg.attr = cef_add_attr(cef_msg.attr, "cs4Label=Command cs4=", auparse_find_field(au, "comm"));
                                goto_record_type(au, type);

                                if (auparse_find_field(au, "parent"))
                                        cef_msg.attr = cef_add_attr(cef_msg.attr, "sproc=", get_proc_name(auparse_get_field_int(au)));
                                goto_record_type(au, type);

                                if (auparse_find_field(au, "pid"))
                                        cef_msg.attr = cef_add_attr(cef_msg.attr, "dproc=", get_proc_name(auparse_get_field_int(au)));
                                goto_record_type(au, type);

                                add_extra_record(au, extra, "error");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "name");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "srcname");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "flags");
                                goto_record_type(au, type);
								*/
                                break;
                        case AUDIT_EXECVE:
								/*
								 * Follow up on this one!!!!!!!
								 */
                                argc = auparse_find_field(au, "argc");
                                havecef = 1;
                                cef_msg.msgname = "EXECVE";
								cef_msg.msgdesc = "Triggered to record arguments of the execve system call";
                                if (argc)
                                        argcount = auparse_get_field_int(au);
                                else
                                        argcount = 0;
                                fullcmd[0] = '\0';
                                len = 0;
                                for (i = 0; i != argcount; i++) {
                                        goto_record_type(au, type);
                                        tmplen = snprintf(f, 7, "a%d", i);
                                        f[tmplen] = '\0';
                                        cmd = auparse_find_field(au, f);
                                        cmd = auparse_interpret_field(au);
                                        if (!cmd)
                                                continue;
                                        if (MAX_ARG_LEN-strlen(fullcmd) > strlen(cmd)) {
                                                if (len == 0)
                                                        len += sprintf(fullcmd+len, "%s", cmd);
                                                else
                                                        len += sprintf(fullcmd+len, " %s", cmd);
                                        } else
                                                        strncpy(fullcmdt, "Yes\0", 4);
                                }
                                cef_msg.attr = cef_add_attr(cef_msg.attr, "cs2Label=Truncated cs2=", fullcmdt);
                                cef_msg.attr = cef_add_attr(cef_msg.attr, "cs1Label=Command cs1=", fullcmd);
                                break;
                        case AUDIT_CWD:
                                cwd = auparse_find_field(au, "cwd");
                                havecef = 1;
                                cef_msg.msgname = "CWD";
								cef_msg.msgdesc = "Triggered to record the current working directory";
								/*
                                if (cwd) {
                                        auparse_interpret_field(au);
                                        add_extra_record(au, extra, "cwd");
                                }
								*/
                                break;
                        case AUDIT_PATH:
                                havecef = 1;
                                cef_msg.msgname = "PATH";
								cef_msg.msgdesc = "Triggered to record file name path information";
                                /*
								cef_msg.attr = cef_add_attr(cef_msg.attr, "fname=", auparse_find_field(au, "name"));
								goto_record_type(au, type);

								add_extra_record(au, extra, "inode");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "dev");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "mode");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "ouid");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "ogid");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "rdev");
                                goto_record_type(au, type);
								*/
                                break;
                        case AUDIT_SYSCALL:
								// Auditd Log Example: type=SYSCALL msg=audit(1487578281.177:240575): arch=c000003e syscall=54 success=yes exit=0 a0=3 a1=107 a2=1 a3=7fffab4f6210 items=0 ppid=42253 pid=43664 auid=70673 uid=0 gid=0 euid=0 suid=0 fsuid=0 egid=0 sgid=0 fsgid=0 tty=pts0 ses=10118 comm="tcpdump" exe="/usr/sbin/tcpdump" key=(null)
                                syscall = auparse_find_field(au, "syscall");
                                if (!syscall) {
                                        cef_del_attrs(cef_msg.attr);
                                        return;
                                }
                                i = auparse_get_field_int(au);
                                sys = audit_syscall_to_name(i, machine);
                                if (!sys) {
                                        syslog(LOG_DEBUG, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Unknown system call %u",CEF_AUDIT_MESSAGE_VERSION, i); //EXTS
										//syslog(LOG_DEBUG, "System call %u is not supported by %s", i, PROGRAM_NAME); //EXTS
                                        cef_del_attrs(cef_msg.attr);
                                        return;
                                }

                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside AUDIT_SYSCALL & syscall is: %i, and sys is: %s",CEF_AUDIT_MESSAGE_VERSION,syscall, sys);
                                //if (!strncmp(sys, "write", 5) || !strncmp(sys, "open", 4) || !strncmp(sys, "unlink", 6)) { //EXTS
								if (!strncmp(sys, "write", 5) || !strncmp(sys, "open", 4) || !strncmp(sys, "unlink", 6) || !strncmp(sys, "rename", 6)) { //EXTS
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "WRITE";
                                        //cef_msg.msgdesc = "Write or append to file"; //EXTS
										cef_msg.msgdesc = "Write: %s", unescape(path); //EXTS
                                } else if (!strncmp(sys, "setxattr", 8)) {
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "ATTR";
                                        //cef_msg.msgdesc = "Change file attributes"; //EXTS
										cef_msg.msgdesc = "Attribute: %s", unescape(path); //EXTS
                                } else if (!strncmp(sys, "chmod", 5)) {
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "CHMOD";
                                        //cef_msg.msgdesc = "Change file mode"; //EXTS
										cef_msg.msgdesc = "Chmod: %s", unescape(path); //EXTS
                                } else if (!strncmp(sys, "chown", 5) || !strncmp(sys, "fchown", 6)) { //EXTS
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "CHOWN";
                                        //cef_msg.msgdesc = "Change file owner"; //EXTS
										cef_msg.msgdesc = "Chown: %s", unescape(path); //EXTS
                                } else if (!strncmp(sys, "ptrace",  6)) {
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "PTRACE";
                                        cef_msg.msgdesc = "Process tracing";
								/*
                                } else if (!strncmp(sys, "execve", 6)) {
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "EXECVE";
                                        cef_msg.msgdesc = "Unix Exec";
								*/
                                } else if (!strncmp(sys, "adjtimex", 8)) {
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "ADJTIMEX";
                                        cef_msg.msgdesc = "Time synchronize";
                                } else if (!strncmp(sys, "rename", 7)) {
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "RENAME";
                                        cef_msg.msgdesc = "rename";
                                } else if (!strncmp(sys, "mknod", 5)) {
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "MKNOD";
                                        cef_msg.msgdesc = "mknod";
                                } else if (!strncmp(sys, "rmdir", 5)) {
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "RMDIR";
                                        cef_msg.msgdesc = "rmdir";
                                } else if (!strncmp(sys, "symlink", 7)) {
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "SYMLINK";
                                        cef_msg.msgdesc = "Symbolic link";
								} else if (!strncmp(sys, "ioctl", 5)) { //EXTS
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "promiscuous"; //EXTS
                                        cef_msg.msgdesc = "Promisc: Interface %s set promiscous %s", unescape(dev), promisc ? "on": "off"; //EXTS
                                } else if (!strncmp(sys, "setsockopt", 10)) { //EXTS
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "SETSOCKOPT"; //EXTS
                                        cef_msg.msgdesc = "Set socket option"; //EXTS
										//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside AUDIT_SYSCALL - setsockopt & syscall is: %i, and sys is: %s And havecef is: %i And msgname is: %s",CEF_AUDIT_MESSAGE_VERSION,syscall, sys, havecef, cef_msg.msgname);
                                } else if (!strncmp(sys, "close", 5)) { //EXTS
										// Auditd Log Example:
										subsys = 1;
										havecef = i;
                                        cef_msg.msgname = "CLOSE"; //EXTS
                                        cef_msg.msgdesc = "Close"; //EXTS
								} else {
                                        //syslog(LOG_INFO, "Unhandled system call %u %s", i, sys); //EXTS
										syslog(LOG_INFO, "System call %u %s is not supported by %s", i, sys, PROGRAM_NAME); //EXTS
                                }

                                if (!subsys) {
									havecef = 1;
									cef_msg.msgname = "SYSCALL"; //EXTS
                                    cef_msg.msgdesc = "Triggered to record a system call to the kernel"; //EXTS
								}

                                /*
								cef_msg.attr = cef_add_attr(cef_msg.attr, "cs3Label=AuditKey cs3=", auparse_find_field(au, "key"));
                                goto_record_type(au, type);

                                if (auparse_find_field(au, "ppid"))
                                        cef_msg.attr = cef_add_attr(cef_msg.attr, "cs5Label=ParentProcess cs5=", get_proc_name(auparse_get_field_int(au)));
                                goto_record_type(au, type);

                                if (auparse_find_field(au, "auid")) {
                                        cef_msg.attr = cef_add_attr(cef_msg.attr, "suser=", get_username(auparse_get_field_int(au)));
                                        cef_msg.attr = cef_add_attr(cef_msg.attr, "cn1Label=auid cn1=",  auparse_get_field_str(au));
                                }
                                goto_record_type(au, type);

                                if (auparse_find_field(au, "uid")) {
                                        cef_msg.attr = cef_add_attr(cef_msg.attr, "duser=", get_username(auparse_get_field_int(au)));
                                        cef_msg.attr = cef_add_attr(cef_msg.attr, "duid=", auparse_get_field_str(au));
                                }
                                goto_record_type(au, type);

                                cef_msg.attr = cef_add_attr(cef_msg.attr, "cs4Label=TTY cs4=", auparse_find_field(au, "tty"));
                                goto_record_type(au, type);
                                cef_msg.attr = cef_add_attr(cef_msg.attr, "dproc=", auparse_find_field(au, "exe"));
                                goto_record_type(au, type);

                                add_extra_record(au, extra, "pid");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "gid");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "euid");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "suid");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "fsuid");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "egid");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "sgid");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "fsgid");
                                goto_record_type(au, type);
                                add_extra_record(au, extra, "ses");
                                goto_record_type(au, type);
								*/
                                break;
                        //EXTS
						case AUDIT_USER_ACCT:
                                user_acct = auparse_find_field(au, "USER_ACCT");
								// Auditd Log Example: type=USER_ACCT msg=audit(1487750472.042:331329): user pid=54878 uid=0 auid=4294967295 ses=4294967295 msg=' op=PAM:accounting acct="REDACTED" exe="/usr/sbin/sshd" hostname=REDACTED addr=REDACTED terminal=ssh res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside USER_ACCT & user_acct is: %i",CEF_AUDIT_MESSAGE_VERSION,user_acct);
                                havecef = 1;
                                cef_msg.msgname = "USER_ACCT";
								cef_msg.msgdesc = "Triggered when a user-space user account is modified";

                                break;
                        case AUDIT_USER_AUTH:
                                user_auth = auparse_find_field(au, "USER_AUTH");
								// Auditd Log Example: type=USER_AUTH msg=audit(1487750457.809:331190): user pid=54813 uid=0 auid=4294967295 ses=4294967295 msg='op=PAM:authentication acct="000MGTT-SCOMLXMON" exe="/opt/microsoft/scx/bin/omiserver" hostname=? addr=? terminal=? res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside USER_AUTH & user_auth is: %i",CEF_AUDIT_MESSAGE_VERSION,user_auth);
                                havecef = 1;
                                cef_msg.msgname = "USER_AUTH";
								cef_msg.msgdesc = "Triggered when a user-space authentication attempt is detected";

                                break;
                        case AUDIT_USER_AVC:
                                user_avc = auparse_find_field(au, "USER_AVC");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside USER_AVC & user_avc is: %i",CEF_AUDIT_MESSAGE_VERSION,user_avc);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|USER_AVC|Triggered when a user-space AVC message is generated|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "USER_AVC";
								cef_msg.msgdesc = "Triggered when a user-space AVC message is generated";
                                break;
                        case AUDIT_USER_CHAUTHTOK:
                                user_chauthtok = auparse_find_field(au, "USER_CHAUTHTOK");
								// Auditd Log Example: type=USER_CHAUTHTOK msg=audit(1488205350.827:229220): user pid=28246 uid=0 auid=70673 ses=9123 msg='op=updating-password id=50601 exe="/usr/sbin/usermod" hostname=? addr=? terminal=pts/4 res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside USER_CHAUTHTOK & user_chauthtok is: %i",CEF_AUDIT_MESSAGE_VERSION,user_chauthtok);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|USER_CHAUTHTOK|Triggered when a user account attribute is modified|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "USER_CHAUTHTOK";
								cef_msg.msgdesc = "Triggered when a user account attribute is modified";
                                break;
                        case AUDIT_USER_CMD:
								// FIX THIS FOR THE NEW AUDITD TO CEF MAPPING
                                user_cmd = auparse_find_field(au, "USER_CMD");
								// Auditd Log Example: type=USER_CMD msg=audit(1487750000.590:330995): user pid=54310 uid=70673 auid=70673 ses=13946 msg='cwd="/home/REDACTED" cmd=7375202D terminal=pts/0 res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside USER_CMD & user_cmd is: %i",CEF_AUDIT_MESSAGE_VERSION,user_cmd);
                                havecef = 1;
                                cef_msg.msgname = "USER_CMD";
								cef_msg.msgdesc = "Triggered when a user-space shell command is executed";

                                break;
                        case AUDIT_USER_END:
                                user_end = auparse_find_field(au, "USER_END");
								// Auditd Log Example: type=USER_END msg=audit(1487750401.871:331189): user pid=54782 uid=0 auid=0 ses=13954 msg='op=PAM:session_close acct="root" exe="/usr/sbin/crond" hostname=? addr=? terminal=cron res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside USER_END & user_end is: %i",CEF_AUDIT_MESSAGE_VERSION,user_end);
                                havecef = 1;
                                cef_msg.msgname = "USER_END";
								cef_msg.msgdesc = "Triggered when a user-space session is terminated";

                                break;
                        case AUDIT_USER_ERR:
                                user_err = auparse_find_field(au, "USER_ERR");
								// Auditd Log Example: type=USER_ERR msg=audit(1487681872.091:295198): user pid=64059 uid=0 auid=4294967295 ses=4294967295 msg='op=PAM:bad_ident acct="?" exe="/usr/sbin/sshd" hostname=REDACTED addr=REDACTED terminal=ssh res=failed'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside USER_ERR & user_err is: %i",CEF_AUDIT_MESSAGE_VERSION,user_err);
                                havecef = 1;
                                cef_msg.msgname = "USER_ERR";
								cef_msg.msgdesc = "Triggered when a user account state error is detected";

                                break;
                        case AUDIT_USER_LOGIN:
                                user_login = auparse_find_field(au, "USER_LOGIN");
								// Auditd Log Example: type=USER_LOGIN msg=audit(1487750472.071:331339): user pid=54878 uid=0 auid=12001 ses=13961 msg='op=login id=12001 exe="/usr/sbin/sshd" hostname=REDACTED addr=REDACTED terminal=ssh res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside USER_LOGIN & user_login is: %i",CEF_AUDIT_MESSAGE_VERSION,user_login);
                                havecef = 1;
                                cef_msg.msgname = "USER_LOGIN";
								cef_msg.msgdesc = "Triggered when a user logs in";

                                break;
                        case AUDIT_USER_LOGOUT:
                                user_logout = auparse_find_field(au, "USER_LOGOUT");
								// Auditd Log Example: type=USER_LOGOUT msg=audit(1487750472.088:331342): user pid=54878 uid=0 auid=12001 ses=13961 msg='op=login id=12001 exe="/usr/sbin/sshd" hostname=REDACTED addr=REDACTED terminal=ssh res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside USER_LOGOUT & user_logout is: %i",CEF_AUDIT_MESSAGE_VERSION,user_logout);
                                havecef = 1;
                                cef_msg.msgname = "USER_LOGOUT";
								cef_msg.msgdesc = "Triggered when a user logs out";

                                break;
                        case AUDIT_USER_MGMT:
                                user_mgmt = auparse_find_field(au, "USER_MGMT");
								// Auditd Log Example: type=USER_MGMT msg=audit(1488205352.899:229226): user pid=28257 uid=0 auid=70673 ses=9123 msg='op=deleting-home-directory id=50601 exe="/usr/sbin/userdel" hostname=? addr=? terminal=pts/4 res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside USER_MGMT & user_mgmt is: %i",CEF_AUDIT_MESSAGE_VERSION,user_mgmt);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|USER_MGMT|Triggered to record user-space management data|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "USER_MGMT";
								cef_msg.msgdesc = "Triggered to record user-space management data";

                                break;
                        case AUDIT_GRP_MGMT:
                                user_mgmt = auparse_find_field(au, "GRP_MGMT");
								// Auditd Log Example: type=GRP_MGMT msg=audit(1487945319.710:93011): user pid=14977 uid=0 auid=70673 ses=3548 msg='op=add-shadow-group id=501 exe="/usr/sbin/groupadd" hostname=? addr=? terminal=pts/2 res=success'
								// Auditd Log Example: type=GRP_MGMT msg=audit(1488541937.860:405846): user pid=39481 uid=0 auid=70673 ses=17069 msg='op=changing-group grp="xxxxxxX" gid=501 new_group="xxxxxx1" acct="xxxxxxX" exe="/usr/sbin/groupmod" hostname=? addr=? terminal=pts/1 res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside USER_MGMT & user_mgmt is: %i",CEF_AUDIT_MESSAGE_VERSION,user_mgmt);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|GRP_MGMT|Triggered to record user-space management data|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "GRP_MGMT";
								cef_msg.msgdesc = "Triggered to record user-space management data";

                                break;
                        case AUDIT_USER_START:
                                user_start = auparse_find_field(au, "USER_START");
								// Auditd Log Example: type=USER_START msg=audit(1487768475.236:824): user pid=4481 uid=0 auid=12001 ses=35 msg='op=PAM:session_open acct="REDACTED" exe="/usr/sbin/sshd" hostname=REDACTED addr=REDACTED terminal=ssh res=success'
								// Auditd Log Example: type=USER_START msg=audit(1487768475.415:855): user pid=4491 uid=0 auid=12001 ses=36 msg='op=login id=12001 exe="/usr/sbin/sshd" hostname=REDACTED addr=REDACTED terminal=ssh res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside USER_START & user_start is: %i",CEF_AUDIT_MESSAGE_VERSION,user_start);
                                havecef = 1;
                                cef_msg.msgname = "USER_START";
								cef_msg.msgdesc = "Triggered when a user-space session is started";

                                break;
                        case AUDIT_USER_TTY:
                                user_tty = auparse_find_field(au, "USER_TTY");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside USER_TTY & user_tty is: %i",CEF_AUDIT_MESSAGE_VERSION,user_tty);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|USER_TTY|Triggered when an explanatory message about TTY input to an administrative process is sent from user-space|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "USER_TTY";
								cef_msg.msgdesc = "Triggered when an explanatory message about TTY input to an administrative process is sent from user-space";

                                break;
                        case AUDIT_ADD_GROUP:
                                add_group = auparse_find_field(au, "ADD_GROUP");
								// Auditd Log Example: type=ADD_GROUP msg=audit(1487945319.704:93010): user pid=14977 uid=0 auid=70673 ses=3548 msg='op=add-group id=501 exe="/usr/sbin/groupadd" hostname=? addr=? terminal=pts/2 res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside ADD_GROUP & add_group is: %i",CEF_AUDIT_MESSAGE_VERSION,add_group);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|ADD_GROUP|Triggered when a user-space group is added|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "ADD_GROUP";
								cef_msg.msgdesc = "Triggered when a user-space group is added";

                                break;
                        case AUDIT_ADD_USER:
                                add_user = auparse_find_field(au, "ADD_USER");
								// Auditd Log Example: type=ADD_USER msg=audit(1488205345.698:229213): user pid=28214 uid=0 auid=70673 ses=9123 msg='op=add-user id=50601 exe="/usr/sbin/useradd" hostname=? addr=? terminal=pts/4 res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside ADD_USER & add_user is: %i",CEF_AUDIT_MESSAGE_VERSION,add_user);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|ADD_USER|Triggered when a user-space user account is added|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "ADD_USER";
								cef_msg.msgdesc = "Triggered when a user-space user account is added";

                                break;
                        case AUDIT_DEL_GROUP:
                                del_group = auparse_find_field(au, "DEL_GROUP");
								// Auditd Log Example: type=DEL_GROUP msg=audit(1487945635.748:93176): user pid=15317 uid=0 auid=70673 ses=3548 msg='op=delete-group id=501 exe="/usr/sbin/groupdel" hostname=? addr=? terminal=pts/2 res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside DEL_GROUP & del_group is: %i",CEF_AUDIT_MESSAGE_VERSION,del_group);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|DEL_GROUP|Triggered when a user-space group is deleted|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "DEL_GROUP";
								cef_msg.msgdesc = "Triggered when a user-space group is deleted";

                                break;
                        case AUDIT_DEL_USER:
                                del_user = auparse_find_field(au, "DEL_USER");
								// Auditd Log Example: type=DEL_USER msg=audit(1488205352.899:229224): user pid=28257 uid=0 auid=70673 ses=9123 msg='op=delete-user id=50601 exe="/usr/sbin/userdel" hostname=? addr=? terminal=pts/4 res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside DEL_USER & del_user is: %i",CEF_AUDIT_MESSAGE_VERSION,del_user);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|DEL_USER|Triggered when a user-space user is deleted|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "DEL_USER";
								cef_msg.msgdesc = "Triggered when a user-space user is deleted";

								break;
                        case AUDIT_CHGRP_ID:
                                chgrp_id = auparse_find_field(au, "CHGRP_ID");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CHGRP_ID & chgrp_id is: %i",CEF_AUDIT_MESSAGE_VERSION,chgrp_id);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CHGRP_ID|Triggered when a user-space group ID is changed|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "CHGRP_ID";
								cef_msg.msgdesc = "Triggered when a user-space group ID is changed";

								break;
                        case AUDIT_CHUSER_ID:
                                chuser_id = auparse_find_field(au, "CHUSER_ID");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CHUSER_ID & chuser_id is: %i",CEF_AUDIT_MESSAGE_VERSION,chuser_id);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CHUSER_ID|Triggered when a user-space user ID is changed|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "CHUSER_ID";
								cef_msg.msgdesc = "Triggered when a user-space user ID is changed";

								break;
						case AUDIT_CRED_ACQ:
                                cred_acq = auparse_find_field(au, "CRED_ACQ");
								// Auditd Log Example: type=CRED_ACQ msg=audit(1487762354.840:337667): user pid=65223 uid=0 auid=70673 ses=13981 msg='op=PAM:setcred acct="root" exe="/bin/su" hostname=? addr=? terminal=pts/3 res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CRED_ACQ & cred_acq is: %i",CEF_AUDIT_MESSAGE_VERSION,cred_acq);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CRED_ACQ|Triggered when a user acquires user-space credentials|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
                                havecef = 1;
                                cef_msg.msgname = "CRED_ACQ";
								cef_msg.msgdesc = "Triggered when a user acquires user-space credentials";

                                break;
                        case AUDIT_CRED_DISP:
                                cred_disp = auparse_find_field(au, "CRED_DISP");
								// Auditd Log Example: type=CRED_DISP msg=audit(1487768771.540:1010): user pid=4728 uid=0 auid=12001 ses=42 msg='op=PAM:setcred acct="REDACTED" exe="/usr/sbin/sshd" hostname=REDACTED addr=REDACTED terminal=ssh res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CRED_DISP & cred_disp is: %i",CEF_AUDIT_MESSAGE_VERSION,cred_disp);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CRED_DISP|Triggered when a user disposes of user-space credentials|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
                                havecef = 1;
                                cef_msg.msgname = "CRED_DISP";
								cef_msg.msgdesc = "Triggered when a user disposes of user-space credentials";

                                break;
                        case AUDIT_CRED_REFR:
                                cred_refr = auparse_find_field(au, "CRED_REFR");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CRED_REFR & cred_refr is: %i",CEF_AUDIT_MESSAGE_VERSION,cred_refr);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CRED_REFR|Triggered when a user refreshes their user-space credentials|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "CRED_REFR";
								cef_msg.msgdesc = "Triggered when a user refreshes their user-space credentials";

                                break;
                        case AUDIT_CRYPTO_FAILURE_USER:
                                crypto_failure_user = auparse_find_field(au, "CRYPTO_FAILURE_USER");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CRYPTO_FAILURE_USER & crypto_failure_usercrypto_failure_user is: %i",CEF_AUDIT_MESSAGE_VERSION,crypto_failure_user);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CRYPTO_FAILURE_USER|Triggered when a decrypt, encrypt, or randomize cryptographic operation fails|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "CRYPTO_FAILURE_USER";
								cef_msg.msgdesc = "Triggered when a decrypt, encrypt, or randomize cryptographic operation fails";

                                break;
                        case AUDIT_CRYPTO_KEY_USER:
                                crypto_key_user = auparse_find_field(au, "CRYPTO_KEY_USER");
								// Auditd Log Example: type=CRYPTO_KEY_USER msg=audit(1487768771.542:1014): user pid=4728 uid=0 auid=12001 ses=42 msg='op=destroy kind=server fp=e6:f9:e9:4e:c9:48:c1:1b:d5:b4:ee:f4:71:85:76:d3 direction=? spid=4728 suid=0  exe="/usr/sbin/sshd" hostname=? addr=REDACTED terminal=? res=success'
								// Auditd Log Example: type=CRYPTO_KEY_USER msg=audit(1487589369.122:246353): user pid=52468 uid=0 auid=12001 ses=10402 msg='op=destroy kind=session fp=? direction=both spid=52470 suid=12001 rport=41668 laddr=10.97.166.15 lport=22  exe="/usr/sbin/sshd" hostname=? addr=REDACTED terminal=? res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CRYPTO_KEY_USER & crypto_key_user is: %i",CEF_AUDIT_MESSAGE_VERSION,crypto_key_user);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CRYPTO_KEY_USER|Triggered to record the cryptographic key identifier used for cryptographic purposes|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "CRYPTO_KEY_USER";
								cef_msg.msgdesc = "Triggered to record the cryptographic key identifier used for cryptographic purposes";

                                break;
                        case AUDIT_CRYPTO_LOGIN:
                                crypto_login = auparse_find_field(au, "CRYPTO_LOGIN");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CRYPTO_LOGIN & crypto_login is: %i",CEF_AUDIT_MESSAGE_VERSION,crypto_login);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CRYPTO_LOGIN|Triggered when a cryptographic officer login attempt is detected|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "CRYPTO_LOGIN";
								cef_msg.msgdesc = "Triggered when a cryptographic officer login attempt is detected";

                                break;
                        case AUDIT_CRYPTO_LOGOUT:
                                crypto_logout = auparse_find_field(au, "CRYPTO_LOGOUT");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CRYPTO_LOGOUT & crypto_logout is: %i",CEF_AUDIT_MESSAGE_VERSION,crypto_logout);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CRYPTO_LOGOUT|Triggered when a crypto officer logout attempt is detected|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "CRYPTO_LOGOUT";
								cef_msg.msgdesc = "Triggered when a crypto officer logout attempt is detected";

                                break;
                        case AUDIT_CRYPTO_PARAM_CHANGE_USER:
                                crypto_param_change_user = auparse_find_field(au, "CRYPTO_PARAM_CHANGE_USER");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CRYPTO_PARAM_CHANGE_USER & crypto_param_change_user is: %i",CEF_AUDIT_MESSAGE_VERSION,crypto_param_change_user);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CRYPTO_PARAM_CHANGE_USER|Triggered when a change in a cryptographic parameter is detected|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "CRYPTO_PARAM_CHANGE_USER";
								cef_msg.msgdesc = "Triggered when a change in a cryptographic parameter is detected";

                                break;
                        case AUDIT_CRYPTO_REPLAY_USER:
                                crypto_replay_user = auparse_find_field(au, "CRYPTO_REPLAY_USER");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CRYPTO_REPLAY_USER & crypto_replay_user is: %i",CEF_AUDIT_MESSAGE_VERSION,crypto_replay_user);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CRYPTO_REPLAY_USER|Triggered when a replay attack is detected|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "CRYPTO_REPLAY_USER";
								cef_msg.msgdesc = "Triggered when a replay attack is detected";

                                break;
                        case AUDIT_CRYPTO_SESSION:
                                crypto_session = auparse_find_field(au, "CRYPTO_SESSION");
								// Auditd Log Example: type=CRYPTO_SESSION msg=audit(1487768771.409:993): user pid=4728 uid=0 auid=4294967295 ses=4294967295 msg='op=start direction=from-server cipher=aes128-ctr ksize=128 spid=4729 suid=74 rport=59158 laddr=10.97.166.15 lport=22  exe="/usr/sbin/sshd" hostname=? addr=REDACTED terminal=? res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CRYPTO_SESSION & crypto_session is: %i",CEF_AUDIT_MESSAGE_VERSION,crypto_session);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CRYPTO_SESSION|Triggered to record parameters set during a TLS session establishment|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "CRYPTO_SESSION";
								cef_msg.msgdesc = "Triggered to record parameters set during a TLS session establishment";

                                break;
                        case AUDIT_CRYPTO_TEST_USER:
                                crypto_test_user = auparse_find_field(au, "CRYPTO_TEST_USER");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CRYPTO_TEST_USER & crypto_test_user is: %i",CEF_AUDIT_MESSAGE_VERSION,crypto_test_user);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CRYPTO_TEST_USER|Triggered to record cryptographic test results as required by the FIPS-140 standard|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "CRYPTO_TEST_USER";
								cef_msg.msgdesc = "Triggered to record cryptographic test results as required by the FIPS-140 standard";

                                break;
						case AUDIT_TRUSTED_APP:
                                trusted_app = auparse_find_field(au, "TRUSTED_APP");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside TRUSTED_APP & trusted_app is: %i",CEF_AUDIT_MESSAGE_VERSION,trusted_app);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|TRUSTED_APP|The record of this type can be used by third party application that require auditing|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "TRUSTED_APP";
								cef_msg.msgdesc = "The record of this type can be used by third party application that require auditing";

                                break;
						case AUDIT_SYSTEM_SHUTDOWN:
                                system_shutdown = auparse_find_field(au, "SYSTEM_SHUTDOWN");
								// Auditd Log Example: type=SYSTEM_SHUTDOWN msg=audit(1487767044.843:340646): user pid=6105 uid=0 auid=70673 ses=13946 msg='init exe="/sbin/shutdown" hostname=? addr=? terminal=pts/0 res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside SYSTEM_SHUTDOWN & system_shutdown is: %i",CEF_AUDIT_MESSAGE_VERSION,system_shutdown);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|SYSTEM_SHUTDOWN|Triggered when the system is shut down|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "SYSTEM_SHUTDOWN";
								cef_msg.msgdesc = "Triggered when the system is shut down";

                                break;
						case AUDIT_SYSTEM_RUNLEVEL:
                                system_runlevel = auparse_find_field(au, "SYSTEM_RUNLEVEL");
								// Auditd Log Example: type=SYSTEM_RUNLEVEL msg=audit(1487767044.843:340645): user pid=6105 uid=0 auid=70673 ses=13946 msg='old-level=3 new-level=6 exe="/sbin/shutdown" hostname=? addr=? terminal=pts/0 res=success'
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside SYSTEM_RUNLEVEL & system_runlevel is: %i",CEF_AUDIT_MESSAGE_VERSION,system_runlevel);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|SYSTEM_RUNLEVEL|Triggered when the system's run level is changed|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "SYSTEM_RUNLEVEL";
								cef_msg.msgdesc = "Triggered when the system's run level is changed";

                                break;
                        case AUDIT_SYSTEM_BOOT:
                                system_boot = auparse_find_field(au, "SYSTEM_BOOT");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside SYSTEM_BOOT & system_boot is: %i",CEF_AUDIT_MESSAGE_VERSION,system_boot);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|SYSTEM_BOOT|Triggered when the system is booted up|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "SYSTEM_BOOT";
								cef_msg.msgdesc = "Triggered when the system is booted up";

                                break;
						case AUDIT_SERVICE_START:
                                service_start = auparse_find_field(au, "SERVICE_START");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside SERVICE_START & service_start is: %i",CEF_AUDIT_MESSAGE_VERSION,service_start);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|SERVICE_START|Triggered when a service is started|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "SERVICE_START";
								cef_msg.msgdesc = "Triggered when a service is started";

                                break;
                        case AUDIT_SERVICE_STOP:
                                service_stop = auparse_find_field(au, "SERVICE_STOP");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside SERVICE_STOP & service_stop is: %i",CEF_AUDIT_MESSAGE_VERSION,service_stop);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|SERVICE_STOP|Triggered when a service is stopped|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "SERVICE_STOP";
								cef_msg.msgdesc = "Triggered when a service is stopped";

                                break;
                        case AUDIT_OBJ_PID:
                                obj_pid = auparse_find_field(au, "OBJ_PID");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside OBJ_PID & obj_pid is: %i",CEF_AUDIT_MESSAGE_VERSION,obj_pid);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|OBJ_PID|Triggered to record information about a process to which a signal is sent|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "OBJ_PID";
								cef_msg.msgdesc = "Triggered to record information about a process to which a signal is sent";

                                break;
                        case AUDIT_LOGIN:
                                login = auparse_find_field(au, "LOGIN");
								// Auditd Log Example: type=LOGIN msg=audit(1487769073.301:1133): pid=4984 uid=0 old auid=4294967295 new auid=12001 old ses=4294967295 new ses=48
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside LOGIN & login is: %i",CEF_AUDIT_MESSAGE_VERSION,login);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|LOGIN|Triggered to record relevant login information when a user log in to access the system|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "LOGIN";
								cef_msg.msgdesc = "Triggered to record relevant login information when a user log in to access the system";

                                break;
                        case AUDIT_KERNEL:
                                kernel = auparse_find_field(au, "KERNEL");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside KERNEL & kernel is: %i",CEF_AUDIT_MESSAGE_VERSION,kernel);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|KERNEL|Triggered to record the initialization of the Audit system|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "KERNEL";
								cef_msg.msgdesc = "Triggered to record the initialization of the Audit system";

                                break;
                        case AUDIT_KERNEL_OTHER:
                                kernel_other = auparse_find_field(au, "KERNEL_OTHER");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside KERNEL_OTHER & kernel_other is: %i",CEF_AUDIT_MESSAGE_VERSION,kernel_other);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|KERNEL_OTHER|Triggered to record information from third-party kernel modules|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "KERNEL_OTHER";
								cef_msg.msgdesc = "Triggered to record information from third-party kernel modules";

                                break;
                        case AUDIT_GRP_AUTH:
                                grp_auth = auparse_find_field(au, "GRP_AUTH");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside GRP_AUTH & grp_auth is: %i",CEF_AUDIT_MESSAGE_VERSION,grp_auth);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|GRP_AUTH|Triggered when a group password is used to authenticate against a user-space group|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "GRP_AUTH";
								cef_msg.msgdesc = "Triggered when a group password is used to authenticate against a user-space group";

                                break;
                        case AUDIT_DAEMON_ABORT:
                                daemon_abort = auparse_find_field(au, "DAEMON_ABORT");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside DAEMON_ABORT & daemon_abort is: %i",CEF_AUDIT_MESSAGE_VERSION,daemon_abort);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|DAEMON_ABORT|Triggered when a daemon is stopped due to an error|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "DAEMON_ABORT";
								cef_msg.msgdesc = "Triggered when a daemon is stopped due to an error";

                                break;
                        case AUDIT_DAEMON_ACCEPT:
                                daemon_accept = auparse_find_field(au, "DAEMON_ACCEPT");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside DAEMON_ACCEPT & daemon_accept is: %i",CEF_AUDIT_MESSAGE_VERSION,daemon_accept);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|DAEMON_ACCEPT|Triggered when the auditd daemon accepts a remote connection|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "DAEMON_ACCEPT";
								cef_msg.msgdesc = "Triggered when the auditd daemon accepts a remote connection";

                                break;
                        case AUDIT_DAEMON_CLOSE:
                                daemon_close = auparse_find_field(au, "DAEMON_CLOSE");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside DAEMON_CLOSE & daemon_close is: %i",CEF_AUDIT_MESSAGE_VERSION,daemon_close);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|DAEMON_CLOSE|Triggered when the auditd daemon closes a remote connection|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "DAEMON_CLOSE";
								cef_msg.msgdesc = "Triggered when the auditd daemon closes a remote connection";

                                break;
                        case AUDIT_DAEMON_CONFIG:
                                daemon_config = auparse_find_field(au, "DAEMON_CONFIG");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside DAEMON_CONFIG & daemon_config is: %i",CEF_AUDIT_MESSAGE_VERSION,daemon_config);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|DAEMON_CONFIG|Triggered when a daemon configuration change is detected|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "DAEMON_CONFIG";
								cef_msg.msgdesc = "Triggered when a daemon configuration change is detected";

                                break;
                        case AUDIT_DAEMON_END:
                                daemon_end = auparse_find_field(au, "DAEMON_END");
								// Auditd Log Example: type=DAEMON_END msg=audit(1487769102.663:7489): auditd normal halt, sending auid=70673 pid=5026 subj= res=success
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside DAEMON_END & daemon_end is: %i",CEF_AUDIT_MESSAGE_VERSION,daemon_end);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|DAEMON_END|Triggered when a daemon is successfully stopped|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "DAEMON_END";
								cef_msg.msgdesc = "Triggered when a daemon is successfully stopped";

                                break;
                        case AUDIT_DAEMON_RESUME:
                                daemon_resume = auparse_find_field(au, "DAEMON_RESUME");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside DAEMON_RESUME & daemon_resume is: %i",CEF_AUDIT_MESSAGE_VERSION,daemon_resume);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|DAEMON_RESUME|Triggered when the auditd daemon resumes logging|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
                                havecef = 1;
                                cef_msg.msgname = "DAEMON_RESUME";
								cef_msg.msgdesc = "Triggered when the auditd daemon resumes logging";

                                break;
                        case AUDIT_DAEMON_ROTATE:
                                daemon_rotate = auparse_find_field(au, "DAEMON_ROTATE");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside DAEMON_ROTATE & daemon_rotate is: %i",CEF_AUDIT_MESSAGE_VERSION,daemon_rotate);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|DAEMON_ROTATE|Triggered when the auditd daemon rotates the Audit log files|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "DAEMON_ROTATE";
								cef_msg.msgdesc = "Triggered when the auditd daemon rotates the Audit log files";

                                break;
                        case AUDIT_DAEMON_START:
                                daemon_start = auparse_find_field(au, "DAEMON_START");
								// Auditd Log Example: type=DAEMON_START msg=audit(1487769102.777:209): auditd start, ver=2.4.5 format=raw kernel=2.6.32-642.13.1.el6.x86_64 auid=70673 pid=5053 res=success
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside DAEMON_START & daemon_start is: %i",CEF_AUDIT_MESSAGE_VERSION,daemon_start);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|DAEMON_START|Triggered when the auditd daemon is started|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "DAEMON_START";
								cef_msg.msgdesc = "Triggered when the auditd daemon is started";

                                break;
						case AUDIT_CONFIG_CHANGE:
                                config_change = auparse_find_field(au, "CONFIG_CHANGE");
								// Auditd Log Example: type=CONFIG_CHANGE msg=audit(1487769102.882:1178): auid=70673 ses=1 op="add rule" key="jn_setclock" list=4 res=1
								// Auditd Log Example: type=CONFIG_CHANGE msg=audit(1487769102.882:1177): audit_backlog_limit=320 old=320 auid=70673 ses=1 res=1
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CONFIG_CHANGE & config_change is: %i",CEF_AUDIT_MESSAGE_VERSION,config_change);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CONFIG_CHANGE|Triggered when the Audit system configuration is modified|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "CONFIG_CHANGE";
								cef_msg.msgdesc = "Triggered when the Audit system configuration is modified";

                                break;
						case AUDIT_ANOM_PROMISCUOUS:
                                config_change = auparse_find_field(au, "ANOM_PROMISCUOUS");
								// Auditd Log Example: type=ANOM_PROMISCUOUS msg=audit(1488291141.668:274715): dev=eth0 prom=256 old_prom=0 auid=70673 uid=0 gid=0 ses=11012
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside CONFIG_CHANGE & config_change is: %i",CEF_AUDIT_MESSAGE_VERSION,config_change);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|CONFIG_CHANGE|Triggered when the Audit system configuration is modified|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "ANOM_PROMISCUOUS";
								cef_msg.msgdesc = "Triggered when a device enables or disables promiscuous mode";

                                break;
						case AUDIT_TTY:
                                tty = auparse_find_field(au, "TTY");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside TTY & tty is: %i",CEF_AUDIT_MESSAGE_VERSION,tty);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|TTY|Triggered when TTY input was sent to an administrative process|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "TTY";
								cef_msg.msgdesc = "Triggered when TTY input was sent to an administrative process";

                                break;
						case AUDIT_TEST:
                                test = auparse_find_field(au, "TEST");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside TEST & test is: %i",CEF_AUDIT_MESSAGE_VERSION,test);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|TEST|Triggered to record the success value of a test message|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 1;
                                cef_msg.msgname = "TEST";
								cef_msg.msgdesc = "Triggered to record the success value of a test message";

                                break;
						case AUDIT_EOE:
                                eoe = auparse_find_field(au, "EOE");
								// Auditd Log Example:
                                //syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside EOE & eoe is: %i",CEF_AUDIT_MESSAGE_VERSION,test);
								//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|EOE|Triggered to record the end of a multi-record event|1|msg=This Function have not been Implemented yet!",CEF_AUDIT_MESSAGE_VERSION);
								havecef = 0;
                                //cef_msg.msgname = "EOE";
								//cef_msg.msgdesc = "Triggered to record the end of a multi-record event";
                                break;
						//EXTS
                        default:
                                havecef = 1;
                                if (auparse_find_field(au, "type")) {
		                                cef_msg.msgname = auparse_get_field_str(au);
                                }
                                goto_record_type(au, type);
								cef_msg.msgdesc = "Linux Audit Message";
                                break;
                }
				//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|After Switch - havecef is: %i And msgname is: %s",CEF_AUDIT_MESSAGE_VERSION, havecef, cef_msg.msgname);
				if (havecef  > 0) {
					//syslog(LOG_INFO, "CEF:0|Unix|auditd|%i|Information|audisp-cef - Information|1|Inside havecef - havecef is: %i And msgname is: %s",CEF_AUDIT_MESSAGE_VERSION, havecef, cef_msg.msgname);
					/*
					 * This is where we add all the fields to the CEF Message
					 */
					// This is the Audit Events Fields to CEF Fields map
					struct auditd_cef_map_type auditd_cef_map = {
							.pid            		= "dpid=", // process ID
							.uid            		= "duid=", // user ID
							.duser          		= "duser=", // translated user name from the above uid
							.auid           		= "cn1Label=auid cn1=", // login user ID (Auid looks to be present in "all" the events, but we cannont use the suid field since we have events where we also get this)
							.auid_user				= "cs1Label=auid_user cs1=", // look above
							.suser          		= "suser=", // ??
							.ses					= "cn2Label=session cn2=", // login session ID
							.msg					= "msg=", // the payload of the audit record
							.acct					= "cs2Label=account_name cs2=", // a user's account name
							.exe					= "dproc=", // 	executable name
							.hostname				= "shost=", // the hostname that the user is connecting from
							.addr					= "src=", // the remote address that the user is connecting from
							.terminal				= "cs6Label=terminal cs6=", // terminal name the user is running programs on
							.res					= "cs3Label=result cs3=", // result of the audited operation(success/fail)
							.res_outcome			= "outcome=", // result of the audited operation(success/fail)
							.op						= "act=", // the operation being performed that is audited
							.id						= "cn3Label=changed_user_id cn3=", // during account changes (the user ID of the account)
							.cmd					= "dproc=", // command being executed
							.fp						= "filePermission=", // file assigned permitted capability map - crypto key finger print
							.direction				= "cs4Label=direction cs4=", // direction of crypto operation (we cannot use the deviceDirection since it is an integer, and we have text in this field)
							.spid					= "spid=", // sent process ID
							.suid					= "suid=", // sent user ID
							.kind					= "cs5Label=kind cs5=", // server or client in crypto operation
							.rport					= "spt=", // remote port number
							.laddr					= "dst=", // local network address
							.lport					= "dpt=", // local network port
							.cipher					= "cs5Label=cipher cs5=", // name of crypto cipher selected
							.ksize					= "cn5Label=ksize cn5=", // key size for crypto operation
							.old_auid				= "cn2Label=old_auid cn2=", // previous auid value
							.new_auid				= "cn3Label=new_auid cn3=", // new auid value
							.old_ses				= "cn4Label=old_ses cn4=", // previous ses value
							.new_ses				= "cn5Label=new_ses cn5=", // new ses value
							.subj					= "cs5Label=subject cs5=", // lspp subject's context string
							.ver					= "cs5Label=auditd_version cs5=", // audit daemon's version number
							.format					= "cs6Label=auditd_format cs6=", // audit log's format
							.key					= "cs6Label=auditd_key cs6=", // key assigned from triggered audit rule
							.list					= "cn2Label=auditd_filter_list cn2=", // the audit system's filter list number
							.audit_backlog_limit	= "cn3Label=audit_backlog_limit cn3=", // audit system's backlog queue size
							.old					= "cn4Label=audit_backlog_old_limit cn4=", // present value of kernel feature - old value
							.cwd					= "filePath=", // the current working directory
							.name					= "fname=", // file name in avcs
							.inode					= "fileId=", // inode number
							//.ppid					= "cn2Label=parent_process_id cn2=", // parent process ID
							.ppid					= "spid=", // parent process ID
							.pproc					= "sproc=", // parent process
							.tty					= "cs6Label=terminal cs6=", // tty udevice the user is running programs on (Do this also come in events where terminal is also present?)
							//.gid					= "gid=", // group ID
							.gid					= "flexNumber1Label=gid flexNumber1=", // group ID
							.gid_name				= "flexString1Label=gid_name flexString1=", // group ID name
							.grp					= "flexString1Label=group_name flexString1=", // group name
							.new_group				= "flexString2Label=new_group flexString2=", // new group name (hmm looks to have been removed in the field set)
							.euid					= "euid=", // effective user ID
							.fsuid					= "fsuid=", // file system user ID
							.egid					= "egid=", // effective group ID
							.sgid					= "sgid=", // set group ID
							.fsgid					= "fsgid=", // file system group ID
							.dev					= "dev=", // device name as found in /dev
							.mode					= "mode=", // mode flags on a file
							.ouid					= "ouid=", // file owner user ID
							.ogid					= "ogid=", // file owner group ID
							.rdev					= "rdev=", // the device identifier (special files only)
							.fname					= "fname=", // ??
							.error					= "error=", // ??
							.srcname				= "srcname=", // ??
							.flags					= "flags=", // mmap syscall flags
							.comm					= "comm=", // command line program name
							.profile				= "profile=", // ??
							.info					= "info=", // ??
							};
					// This is the Audit Events Fields to CEF Fields map
					/*
					 * This code block will try and get the value of the "user pid" field name
					 * (Should a do while block like this be used for all the fields?)
					 */
					 goto_record_type(au, type);
					 auparse_first_field(au);
					 do {
						 if (auparse_get_field_name(au) == "user pid") {
							 cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.pid, auparse_get_field_str(au));
						 }
					 } while (auparse_next_field(au) > 0);
					 goto_record_type(au, type);
					 /* This is the end of the above code block */

					// This is the start of the pid section
					if (auparse_find_field(au, "pid")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.pid, auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the pid section

					// This is the start of the uid section
					if (auparse_find_field(au, "uid")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.duser, get_username(auparse_get_field_int(au)));
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.uid, auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the uid section

					// This is the start of the auid section
					if (auparse_find_field(au, "auid")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.suser, get_username(auparse_get_field_int(au)));
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.auid,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the auid section

					// This is the start of the ses section
					if (auparse_find_field(au, "ses")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.ses,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the ses section

					// This is the start of the msg section
					if (auparse_find_field(au, "msg")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.msg,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the msg section

					// This is the start of the acct section
					if (auparse_find_field(au, "acct")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.acct,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the acct section

					// This is the start of the exe section
					if (auparse_find_field(au, "exe")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.exe,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the exe section

					// This is the start of the hostname section
					// If this is empty or a ? should we put in the local hostname?
					if (auparse_find_field(au, "hostname")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.hostname,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the hostname section

					// This is the start of the addr section
					// If this is empty or a ? should we put in the local IP address?
					if (auparse_find_field(au, "addr")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.addr,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the addr section

					// This is the start of the terminal section
					if (auparse_find_field(au, "terminal")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.terminal,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the terminal section

					// This is the start of the res section
					if (auparse_find_field(au, "res")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.res,  auparse_get_field_str(au));
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.res_outcome,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the res section

					// This is the start of the op section
					if (auparse_find_field(au, "op")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.op,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the op section

					// This is the start of the cmd section
					if (auparse_find_field(au, "cmd")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.cmd,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the cmd section

					// This is the start of the cwd section
					if (auparse_find_field(au, "cwd")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.cwd,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the cwd section

					// This is the start of the id section
					if (auparse_find_field(au, "id")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.id,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the id section

					// This is the start of the name section
					if (auparse_find_field(au, "name")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.name,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the name section

					// This is the start of the inode section
					if (auparse_find_field(au, "inode")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.inode,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the inode section

					// This is the start of the ppid section
					if (auparse_find_field(au, "ppid")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.ppid,  auparse_get_field_str(au));
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.pproc, get_proc_name(auparse_get_field_int(au)));
					}
					goto_record_type(au, type);
					// This is the stop of the ppid section

					// This is the start of the tty section
					if (auparse_find_field(au, "tty")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.tty,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the tty section

					// This is the start of the kind section
					if (auparse_find_field(au, "kind")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.kind,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the kind section

					// This is the start of the direction section
					if (auparse_find_field(au, "direction")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.direction,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the direction section

					// This is the start of the rport section
					if (auparse_find_field(au, "rport")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.rport,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the rport section

					// This is the start of the laddr section
					if (auparse_find_field(au, "laddr")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.laddr,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the laddr section

					// This is the start of the lport section
					if (auparse_find_field(au, "lport")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.lport,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the lport section

					// This is the start of the cipher section
					if (auparse_find_field(au, "cipher")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.cipher,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the cipher section

					// This is the start of the ksize section
					if (auparse_find_field(au, "ksize")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.ksize,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the ksize section

					/*
					 * This code block will try and get the value of the "old auid" field name
					 * (Should a do while block like this be used for all the fields?)
					 */
					 goto_record_type(au, type);
					 auparse_first_field(au);
					 do {
						 if (auparse_get_field_name(au) == "old auid") {
							 cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.old_auid, auparse_get_field_str(au));
						 }
					 } while (auparse_next_field(au) > 0);
					 goto_record_type(au, type);
					 /* This is the end of the above code block */

					 /*
					 * This code block will try and get the value of the "new auid" field name
					 * (Should a do while block like this be used for all the fields?)
					 */
					 goto_record_type(au, type);
					 auparse_first_field(au);
					 do {
						 if (auparse_get_field_name(au) == "new auid") {
							 cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.new_auid, auparse_get_field_str(au));
						 }
					 } while (auparse_next_field(au) > 0);
					 goto_record_type(au, type);
					 /* This is the end of the above code block */

					 /*
					 * This code block will try and get the value of the "old ses" field name
					 * (Should a do while block like this be used for all the fields?)
					 */
					 goto_record_type(au, type);
					 auparse_first_field(au);
					 do {
						 if (auparse_get_field_name(au) == "old ses") {
							 cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.old_ses, auparse_get_field_str(au));
						 }
					 } while (auparse_next_field(au) > 0);
					 goto_record_type(au, type);
					 /* This is the end of the above code block */

					 /*
					 * This code block will try and get the value of the "new ses" field name
					 * (Should a do while block like this be used for all the fields?)
					 */
					 goto_record_type(au, type);
					 auparse_first_field(au);
					 do {
						 if (auparse_get_field_name(au) == "new ses") {
							 cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.new_ses, auparse_get_field_str(au));
						 }
					 } while (auparse_next_field(au) > 0);
					 goto_record_type(au, type);
					 /* This is the end of the above code block */

					 // This is the start of the subj section
					if (auparse_find_field(au, "subj")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.subj,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the subj section

					// This is the start of the ver section
					if (auparse_find_field(au, "ver")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.ver,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the ver section

					// This is the start of the format section
					if (auparse_find_field(au, "format")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.format,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the format section

					// This is the start of the key section
					if (auparse_find_field(au, "key")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.key,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the key section

					// This is the start of the list section
					if (auparse_find_field(au, "list")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.list,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the list section

					// This is the start of the audit_backlog_limit section
					if (auparse_find_field(au, "audit_backlog_limit")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.audit_backlog_limit,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the audit_backlog_limit section

					// This is the start of the old section
					if (auparse_find_field(au, "old")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.old,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the old section

					// This is the start of the gid section
					if (auparse_find_field(au, "gid")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.gid,  auparse_get_field_str(au));
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.gid_name, get_groupname(auparse_get_field_int(au)));
					}
					goto_record_type(au, type);
					// This is the stop of the gid section

					// This is the start of the grp section
					if (auparse_find_field(au, "grp")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.grp,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the grp section

					// This is the start of the new_group section
					if (auparse_find_field(au, "new_group")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.new_group,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the new_group section

					// This is the start of the euid section
					if (auparse_find_field(au, "euid")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.euid,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the euid section

					// This is the start of the fsuid section
					if (auparse_find_field(au, "fsuid")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.fsuid,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the fsuid section

					// This is the start of the egid section
					if (auparse_find_field(au, "egid")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.egid,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the egid section

					// This is the start of the sgid section
					if (auparse_find_field(au, "sgid")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.sgid,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the sgid section

					// This is the start of the fsgid section
					if (auparse_find_field(au, "fsgid")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.fsgid,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the fsgid section

					// This is the start of the dev section
					if (auparse_find_field(au, "dev")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.dev,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the dev section

					// This is the start of the mode section
					if (auparse_find_field(au, "mode")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.mode,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the mode section

					// This is the start of the ouid section
					if (auparse_find_field(au, "ouid")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.ouid,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the ouid section

					// This is the start of the ogid section
					if (auparse_find_field(au, "ogid")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.ogid,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the ogid section

					// This is the start of the rdev section
					if (auparse_find_field(au, "rdev")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.rdev,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the rdev section

					// This is the start of the fname section
					if (auparse_find_field(au, "fname")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.fname,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the fname section

					// This is the start of the error section
					if (auparse_find_field(au, "error")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.error,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the error section

					// This is the start of the srcname section
					if (auparse_find_field(au, "srcname")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.srcname,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the srcname section

					// This is the start of the flags section
					if (auparse_find_field(au, "flags")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.flags,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the flags section

					// This is the start of the comm section
					if (auparse_find_field(au, "comm")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.comm,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the comm section

					// This is the start of the profile section
					if (auparse_find_field(au, "profile")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.profile,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the profile section

					// This is the start of the info section
					if (auparse_find_field(au, "info")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.info,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the info section

					/*
					// This is the start of the CHANGEME section
					if (auparse_find_field(au, "CHANGEME")) {
							cef_msg.attr = cef_add_attr(cef_msg.attr, auditd_cef_map.CHANGEME,  auparse_get_field_str(au));
					}
					goto_record_type(au, type);
					// This is the stop of the CHANGEME section
					*/
				}

                num++;
        }

        if (!havecef) {
                cef_del_attrs(cef_msg.attr);
                return;
        }

        if (strlen(extra) >= MAX_ARG_LEN) {
                extra[MAX_ARG_LEN] = '\0';
                cef_msg.attr = cef_add_attr(cef_msg.attr, "cs6Label=MsgTruncated cs6=", "Yes");
        }
        cef_msg.attr = cef_add_attr(cef_msg.attr, "msg=", extra);
        cef_msg.attr = cef_add_attr(cef_msg.attr, "dhost=", hostname);
        //This also frees cef_msg.attr
        syslog_cef_msg(cef_msg);
}
